<?php
/**
*
* @package install
* @version $Id$
* @copyright (c) 2006 phpBB Group
* @copyright (c) 2013 prototech
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* NOTE to potential convertor authors. Please use this file to get
* familiar with the structure since we added some bare explanations here.
*
* Since this file gets included more than once on one page you are not able to add functions to it.
* Instead use a functions_ file.
*
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

include($phpbb_root_path . 'config.' . $phpEx);
unset($dbpasswd);

/**
* $convertor_data provides some basic information about this convertor which is
* used on the initial list of convertors and to populate the default settings
*/
$convertor_data = array(
	'forum_name'	=> 'PunBB 1.4.x',
	'version'		=> '0.0.4',
	'phpbb_version'	=> '3.0.11',
	'author'		=> '<a href="https://www.phpbb.com/community/memberlist.php?mode=viewprofile&u=304651">prototech</a>',
	'dbms'			=> $dbms,
	'dbhost'		=> $dbhost,
	'dbport'		=> $dbport,
	'dbuser'		=> $dbuser,
	'dbpasswd'		=> '',
	'dbname'		=> $dbname,
	'table_prefix'	=> '',
	'forum_path'	=> '../forums',
	'author_notes'	=> '',
);

/**
* $tables is a list of the tables (minus prefix) which we expect to find in the
* source forum. It is used to guess the prefix if the specified prefix is incorrect
*/
$tables = array(
	'bans',
	'categories',
	'censoring',
	'config',
	'extensions',
	'extension_hooks',
	'forums',
	'forum_perms',
	'forum_subscriptions',
	'groups',
	'online',
	'posts',
	'ranks',
	'reports',
	'search_cache',
	'search_matches',
	'search_words',
	'subscriptions',
	'topics',
	'users'
);

/**
* $config_schema details how the board configuration information is stored in the source forum.
*
* 'table_format' can take the value 'file' to indicate a config file. In this case array_name
* is set to indicate the name of the array the config values are stored in
* Example of using a file:
* $config_schema = array(
* 	'table_format'	=>	'file',
* 	'filename'	=>	'NAME OF FILE', // If the file is not in the root directory, the path needs to be added with no leading slash
* 	'array_name' => 'NAME OF ARRAY', // Only used if the configuration file stores the setting in an array.
* 	'settings'		=>	array(
*        'board_email' => 'SUPPORT_EMAIL', // target config name => source target name
* 	)
* );
* 'table_format' can be an array if the values are stored in a table which is an assosciative array
* (as per phpBB 2.0.x)
* If left empty, values are assumed to be stored in a table where each config setting is
* a column (as per phpBB 1.x)
*
* In either of the latter cases 'table_name' indicates the name of the table in the database
*
* 'settings' is an array which maps the name of the config directive in the source forum
* to the config directive in phpBB3. It can either be a direct mapping or use a function.
* Please note that the contents of the old config value are passed to the function, therefore
* an in-built function requiring the variable passed by reference is not able to be used. Since
* empty() is such a function we created the function is_empty() to be used instead.
*/
$config_schema = array(
	'table_name'	=>	'config',
	'table_format'	=>	array('conf_name' => 'conf_value'),
	'settings'		=>	array(
		'allow_bbcode'			=> 'p_message_bbcode',
		'allow_emailreuse'		=> 'p_allow_dupe_email',
		'allow_quick_reply'		=> 'o_quickpost',
		'allow_smilies'			=> 'o_smilies',
		'allow_sig'				=> 'o_signatures',
		'allow_sig_bbcode'		=> 'p_sig_bbcode',
		'allow_sig_img'			=> 'p_sig_img_tag',
		'allow_sig_smilies'		=> 'o_smilies_sig',
		'allow_avatar_local'	=> 'o_avatars',
		'allow_avatar_remote'	=> 'o_avatars',
		'allow_avatar_upload'	=> 'o_avatars',
		'board_contact'			=> 'o_admin_email',
		'board_email'			=> 'o_webmaster_email',
		'board_disable'			=> 'o_maintenance',
		'board_disable_msg'		=> 'o_maintenance_message',
		'sitename'				=> 'o_board_title',
		'site_desc'				=> 'o_board_desc',
		'load_db_track'			=> 'o_show_dot',
		'load_jumpbox'			=> 'o_quickjump',
		'load_moderators'		=> 'o_show_moderators',
		'load_online'			=> 'o_users_online',
		'load_online_time'		=> 'seconds_to_minutes(o_timeout_online)',
		'session_length'		=> 'o_timeout_visit',
		'posts_per_page'		=> 'o_disp_posts_default',
		'topics_per_page'		=> 'o_disp_topics_default',
		'max_sig_chars'			=> 'p_sig_length',
		'smtp_delivery'			=> 'is_empty(o_smtp_password)',
		'smtp_host'				=> 'o_smtp_host',
		'smtp_username'			=> 'o_smtp_user',
		'smtp_password'			=> 'o_smtp_password',
		'require_activation'	=> 'punbb_require_activation()',
		'avatar_filesize'		=> 'o_avatars_size',
		'avatar_max_width'		=> 'o_avatars_width',
		'avatar_max_height'		=> 'o_avatars_height',
		'board_timezone'		=> 'floatval(o_default_timezone)',
		'board_dst'				=> 'o_default_dst',
		'gzip_compress'			=> 'o_gzip',
		'board_startdate'		=> 0, // Convertor framework will automatically calculate this
		'max_quote_depth'		=> 'o_quote_depth',
	)
);

/**
* $test_file is the name of a file which is present on the source
* forum which can be used to check that the path specified by the
* user was correct
*/
$test_file = 'userlist.php';

/**
* If this is set then we are not generating the first page of information but getting the conversion information.
*/
if (!$get_info)
{
	// Check for some extensions
	$sql = "SELECT id
		FROM {$convert->src_table_prefix}extensions
		WHERE disabled = 0
			AND (id = 'pun_pm' OR id = 'pun_attachment')";
	$result = $src_db->sql_query($sql);
	$extensions = array();

	while ($row = $src_db->sql_fetchrow($result))
	{
		$extensions[] = $row['id'];
	}
	$src_db->sql_freeresult($result);

	@define('PM_ENABLED', in_array('pun_pm', $extensions));
	@define('ATTACH_ENABLED', in_array('pun_attachment', $extensions));

	// Overwrite maximum avatar width/height
	@define('DEFAULT_AVATAR_X_CUSTOM', get_config_value('o_avatars_width'));
	@define('DEFAULT_AVATAR_Y_CUSTOM', get_config_value('o_avatars_height'));

	// additional table used only during conversion
	@define('USERCONV_TABLE', $table_prefix . 'userconv');

/**
*	Description on how to use the convertor framework.
*
*	'schema' Syntax Description
*		-> 'target'			=> Target Table. If not specified the next table will be handled
*		-> 'primary'		=> Primary Key. If this is specified then this table is processed in batches
*		-> 'query_first'	=> array('target' or 'src', Query to execute before beginning the process
*								(if more than one then specified as array))
*		-> 'function_first'	=> Function to execute before beginning the process (if more than one then specified as array)
*								(This is mostly useful if variables need to be given to the converting process)
*		-> 'test_file'		=> This is not used at the moment but should be filled with a file from the old installation
*
*		// DB Functions
*		'distinct'	=> Add DISTINCT to the select query
*		'where'		=> Add WHERE to the select query
*		'group_by'	=> Add GROUP BY to the select query
*		'left_join'	=> Add LEFT JOIN to the select query (if more than one joins specified as array)
*		'having'	=> Add HAVING to the select query
*
*		// DB INSERT array
*		This one consist of three parameters
*		First Parameter:
*							The key need to be filled within the target table
*							If this is empty, the target table gets not assigned the source value
*		Second Parameter:
*							Source value. If the first parameter is specified, it will be assigned this value.
*							If the first parameter is empty, this only gets added to the select query
*		Third Parameter:
*							Custom Function. Function to execute while storing source value into target table.
*							The functions return value get stored.
*							The function parameter consist of the value of the second parameter.
*
*							types:
*								- empty string == execute nothing
*								- string == function to execute
*								- array == complex execution instructions
*
*		Complex execution instructions:
*		@todo test complex execution instructions - in theory they will work fine
*
*							By defining an array as the third parameter you are able to define some statements to be executed. The key
*							is defining what to execute, numbers can be appended...
*
*							'function' => execute function
*							'execute' => run code, whereby all occurrences of {VALUE} get replaced by the last returned value.
*										The result *must* be assigned/stored to {RESULT}.
*							'typecast'	=> typecast value
*
*							The returned variables will be made always available to the next function to continue to work with.
*
*							example (variable inputted is an integer of 1):
*
*							array(
*								'function1'		=> 'increment_by_one',		// returned variable is 2
*								'typecast'		=> 'string',				// typecast variable to be a string
*								'execute'		=> '{RESULT} = {VALUE} . ' is good';', // returned variable is '2 is good'
*								'function2'		=> 'replace_good_with_bad',				// returned variable is '2 is bad'
*							),
*
*/

	$convertor = array(
		'test_file'				=> 'moderate.php',

		'avatar_path'			=> get_config_value('o_avatars_dir') . '/',
		'smilies_path'			=> 'img/smilies/', // Smilies are hard-coded in the code
		'upload_path'			=> 'extensions/pun_attachment/attachments/',
		'thumbnails'			=> '',

		// We empty some tables to have clean data available
		'query_first'			=> array(
			array('target', $convert->truncate_statement . SEARCH_RESULTS_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDLIST_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDMATCH_TABLE),
			array('target', $convert->truncate_statement . LOG_TABLE),
			array('target', $convert->truncate_statement . PRIVMSGS_TABLE),
			array('target', $convert->truncate_statement . PRIVMSGS_FOLDER_TABLE),
			array('target', $convert->truncate_statement . PRIVMSGS_RULES_TABLE),
			array('target', $convert->truncate_statement . PRIVMSGS_TO_TABLE),
			array('target', $convert->truncate_statement . ATTACHMENTS_TABLE),
		),

		// phpBB2 allowed some similar usernames to coexist which would have the same
		// username_clean in phpBB3 which is not possible, so we'll give the admin a list
		// of user ids and usernames and let him deicde what he wants to do with them
		'execute_first'	=> '
			phpbb_create_userconv_table();
			punbb_insert_forums();
			punbb_add_login_field();
		',

		'execute_last'	=> array('
			add_bots();
		', '
			set_config(\'auth_method\', \'punbb\');
		', '
			punbb_import_banlist_extras();
		', '
			punbb_post_reports();
		', '
			punbb_fix_moved_topics();
		', '
			punbb_update_post_attach_status();
		', '
			punbb_convert_authentication(\'start\');
		', '
			punbb_convert_authentication(\'first\');
		'),

		'schema' => array(
			array(
				'target'	=> USERCONV_TABLE,
				'query_first'   => array('target', $convert->truncate_statement . USERCONV_TABLE),


				array('user_id',				'users.id', 			''),
				array('username_clean',		'users.username',		array('function1' => 'punbb_htmlspecialchars', 'function2' => 'utf8_clean_string')),
			),

			array(
				'target'		=> (ATTACH_ENABLED) ? ATTACHMENTS_TABLE : '',
				'primary'		=> 'attach_files.id',
				'autoincrement'	=> 'attach_id',

				array('attach_id',					'attach_files.id',						''),
				array('post_msg_id',				'attach_files.post_id',					''),
				array('topic_id',					'attach_files.topic_id',				''),
				array('in_message',				0,										''),
				array('is_orphan',					0,										''),
				array('poster_id',					'attach_files.owner_id AS poster_id',	''),
				array('physical_filename',			'attach_files.file_path',				'punbb_import_attachment'),
				array('real_filename',				'attach_files.filename',				''),
				array('download_count',			'attach_files.download_counter',		''),
				array('attach_comment',			'',										''),
				array('extension',					'attach_files.file_ext',				''),
				array('mimetype',					'attach_files.filename',				'mimetype'),
				array('filesize',					'attach_files.size',					''),
				array('filetime',					'attach_files.uploaded_at',				''),
				array('thumbnail',					'',										'punbb_has_thumbnail'),

				'where'			=> 'attach_files.owner_id <> 0',
			),

			array(
				'target'		=> BANLIST_TABLE,
				'execute_first'	=> 'phpbb_check_username_collisions();',
				'query_first'	=> array('target', $convert->truncate_statement . BANLIST_TABLE),

				// The punbb_handle_ban_ip will return an empty string if there are more than one ip's. We'll insert those later.
				array('ban_ip',					'bans.ip',						'punbb_handle_ban_ip'),
				array('ban_userid',				'users.id AS user_id',			'null_to_zero'),
				array('ban_email',					'bans.email',					'punbb_fix_ban_email'),
				array('ban_reason',				'bans.message',					'punbb_htmlspecialchars'),
				array('ban_give_reason',			'bans.message',					'punbb_htmlspecialchars'),
				array('ban_end',					'bans.expire',					'null_to_zero'),

				'left_join'		=> 'bans LEFT JOIN users ON (bans.username = users.username)',
			),

			// Post-based ranks
			array(
				'target'		=> RANKS_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . RANKS_TABLE),
				'autoincrement'	=> 'rank_id',

				array('rank_id',					'ranks.id',						''),
				array('rank_title',				'ranks.rank',					'punbb_htmlspecialchars'),
				array('rank_min',					'ranks.min_posts',				''),
				array('rank_special',				0,								''),
			),

			// Ranks from the user groups
			array(
				'target'		=> RANKS_TABLE,
				'autoincrement'	=> 'rank_id',

				array('rank_title',				'groups.g_user_title',			'punbb_htmlspecialchars'),
				array('rank_special',				1,								''),

				'where'			=> 'groups.g_user_title IS NOT NULL',
			),

			array(
				'target'		=> TOPICS_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . TOPICS_TABLE),
				'primary'		=> 'topics.id',
				'autoincrement'	=> 'topic_id',

				array('topic_id',				'topics.id AS topic_id',			''),
				array('forum_id',				'topics.forum_id',					''),
				array('icon_id',				0,									''),
				array('topic_poster',			'users.id AS poster_id',			''),
				array('topic_attachment',		0,									''),
				array('topic_title',			'topics.subject',					'punbb_htmlspecialchars'),
				array('topic_time',			'topics.posted',					''),
				array('topic_views',			'topics.num_views',					''),
				array('topic_replies',			'topics.num_replies',				''),
				array('topic_replies_real',	'topics.num_replies',				''),
				array('topic_last_post_id',	'topics.last_post_id',				''),
				array('topic_status',			'topics.closed',					''),
				array('topic_moved_id',		0,									''),
				array('topic_type',			'topics.sticky',					''),
				array('topic_first_post_id',	'topics.first_post_id',				''),
				array('topic_last_view_time',	'posts.posted',						''),
				array('poll_title',			'',									''),
				array('poll_start',			0,									''),
				array('poll_length',			0,									''),
				array('poll_max_options',		1,									''),
				array('poll_vote_change',		0,									''),

				'where'			=> 'topics.last_post_id = posts.id AND topics.moved_to IS NULL AND topics.poster = users.username',
			),


			// Dummy moved topic placeholders. These will get fixed in one of the final steps.
			array(
				'target'		=> TOPICS_TABLE,
				'primary'		=> 'topics.id',
				'autoincrement'	=> 'topic_id',

				array('topic_id',				'topics.id AS topic_id',			''),
				array('forum_id',				'topics.forum_id',					''),
				array('topic_title',			'topics.subject',					'punbb_htmlspecialchars'),
				array('topic_time',			'topics.posted',					''),
				array('topic_status',			ITEM_MOVED,							''),
				array('topic_moved_id',		'topics.moved_to',					''),
				array('topic_type',			'topics.sticky',					''),

				'where'			=> 'topics.moved_to IS NOT NULL',
			),

			array(
				'target'		=> TOPICS_WATCH_TABLE,
				'primary'		=> 'subscriptions.topic_id',
				'query_first'	=> array('target', $convert->truncate_statement . TOPICS_WATCH_TABLE),

				array('topic_id',				'subscriptions.topic_id',			''),
				array('user_id',				'subscriptions.user_id',			''),
				array('notify_status',			NOTIFY_YES,							''),
			),

			array(
				'target'		=> FORUMS_WATCH_TABLE,
				'primary'		=> 'forum_subscriptions.forum_id',
				'query_first'	=> array('target', $convert->truncate_statement . FORUMS_WATCH_TABLE),

				array('forum_id',		'forum_subscriptions.forum_id',				''),
				array('user_id',		'forum_subscriptions.user_id',				''),
				array('notify_status',	NOTIFY_YES,									''),
			),

			array(
				'target'		=> WORDS_TABLE,
				'primary'		=> 'censoring.id',
				'query_first'	=> array('target', $convert->truncate_statement . WORDS_TABLE),
				'autoincrement'	=> 'word_id',

				array('word_id',				'censoring.id',						''),
				array('word',					'censoring.search_for',				'punbb_htmlspecialchars'),
				array('replacement',			'censoring.replace_with',			'punbb_htmlspecialchars'),
			),

			array(
				'target'		=> REPORTS_TABLE,
				'primary'		=> 'reports.id',
				'autoincrement'	=> 'report_id',
				'query_first'	=> array('target', $convert->truncate_statement . REPORTS_TABLE),

				array('report_id',				'reports.id',						''),
				array('reason_id',				4,									''), // "Other" reason
				array('post_id',				'reports.post_id',					''),
				array('pm_id',					0,									''),
				array('user_id',				'reports.reported_by',				''),
				array('user_notify',			0,									''),
				array('report_closed',			'reports.zapped',					'not_empty'),
				array('report_time',			'reports.created',					''),
				array('report_text',			'reports.message',					'punbb_htmlspecialchars'),
			),

			array(
				'target'		=> POSTS_TABLE,
				'primary'		=> 'posts.id',
				'autoincrement'	=> 'post_id',
				'query_first'	=> array('target', $convert->truncate_statement . POSTS_TABLE),
				'execute_first'	=> '
					punbb_convert_smilies();
					$config["max_post_chars"] = 0;
					$config["min_post_chars"] = 0;
					$config["max_quote_depth"] = 0;
				',

				array('post_id',				'posts.id AS post_id',				'intval'),
				array('topic_id',				'posts.topic_id',					''),
				array('forum_id',				'topics.forum_id',					''),
				array('poster_id',				'posts.poster_id',					''),
				array('icon_id',				0,									''),
				array('poster_ip',				'posts.poster_ip',					'decode_ip'),
				array('post_time',				'posts.posted AS post_time',		''),
				array('enable_bbcode',			1,									''),
				array('enable_smilies',		'posts.hide_smilies',				'not'),
				array('enable_sig',			1,									''),
				array('enable_magic_url',		'make_link',						'get_config_value'),
				array('post_username',			'posts.poster',						'punbb_guest_name'), // punbb_guest_name() will run htmlspecialchars
				array('post_subject',			'topics.subject',					'punbb_post_subject'), // punbb_post_subject() will run htmlspecialchars
				array('',						'topics.first_post_id',				''),
				array('post_attachment',		0,									''),
				array('post_edit_time',		'posts.edited',						'null_to_zero'),
				array('post_edit_count',		'posts.edited',						'not_empty'),
				array('post_edit_reason',		'',									''),
				array('post_edit_user',		'users.id AS post_edit_user_id',	'null_to_zero'),

				array('bbcode_uid',			'posts.posted',						'make_uid'),
				array('post_text',				'posts.message',					'punbb_prepare_message'),
				array('bbcode_bitfield',		'',									'get_bbcode_bitfield'),
				array('post_checksum',			'',									''),

				'left_join'	=> 'posts LEFT JOIN users ON (posts.edited_by = users.username)',

				'where'		=> 'posts.topic_id = topics.id',
				'group_by'	=> 'posts.id',
			),

			array(
				'target'		=> (PM_ENABLED) ? PRIVMSGS_TABLE : '',
				'primary'		=> 'pun_pm_messages.id',
				'autoincrement'	=> 'msg_id',

				'execute_first'	=> '
					$config["max_post_chars"] = 0;
					$config["min_post_chars"] = 0;
					$config["max_quote_depth"] = 0;
				',

				array('msg_id',				'pun_pm_messages.id',						''),
				array('root_level',			0,											''),
				array('author_id',				'pun_pm_messages.sender_id AS poster_id',	''),
				array('icon_id',				0,											''),
				array('author_ip',				'',											''),
				array('message_time',			'pun_pm_messages.lastedited_at',			''),
				array('enable_bbcode',			1,											''),
				array('enable_smilies',		1,											''),
				array('enable_magic_url',		1,											''),
				array('enable_sig',			1,											''),
				array('message_subject',		'pun_pm_messages.subject',					'punbb_htmlspecialchars'),
				array('message_attachment',	0,											''),
				array('message_edit_reason',	'',											''),
				array('message_edit_user',		0,											''),
				array('message_edit_time',		0,											''),
				array('message_edit_count',	0,											''),

				array('bbcode_uid',			'pun_pm_messages.lastedited_at AS post_time',	'make_uid'),
				array('',						'0 AS hide_smilies',							''),
				array('message_text',			'pun_pm_messages.body',							'punbb_prepare_message'),
				array('bbcode_bitfield',		'',												'get_bbcode_bitfield'),
				array('to_address',			'pun_pm_messages.receiver_id',					'punbb_pm_to_user'),
				array('bcc_address',			'',												''),
			),

			array(
				'target'		=> (PM_ENABLED) ? PRIVMSGS_FOLDER_TABLE : '',
				'primary'		=> 'users.id',

				array('user_id',				'users.id',								''),
				array('folder_name',			$user->lang['CONV_SAVED_MESSAGES'],		''),
				array('pm_count',				0,										''),

				'where'			=> 'users.id <> 1',
			),

			// Inbox
			array(
				'target'		=> (PM_ENABLED) ? PRIVMSGS_TO_TABLE : '',
				'primary'		=> 'pun_pm_messages.id',

				array('msg_id',				'pun_pm_messages.id',						''),
				array('user_id',				'pun_pm_messages.receiver_id',				''),
				array('author_id',				'pun_pm_messages.sender_id',				''),
				array('pm_deleted',			0,											''),
				array('pm_new',				'pun_pm_messages.read_at',					'is_empty'),
				array('pm_unread',				'pun_pm_messages.read_at',					'is_empty'),
				array('pm_replied',			0,											''),
				array('pm_marked',				0,											''),
				array('pm_forwarded',			0,											''),
				array('folder_id',				PRIVMSGS_INBOX,								''),
			),

			// Outbox
			array(
				'target'		=> (PM_ENABLED) ? PRIVMSGS_TO_TABLE : '',
				'primary'		=> 'pun_pm_messages.id',

				array('msg_id',				'pun_pm_messages.id',						''),
				array('user_id',				'pun_pm_messages.sender_id',				''),
				array('author_id',				'pun_pm_messages.sender_id',				''),
				array('pm_deleted',			0,											''),
				array('pm_new',				0,											''),
				array('pm_unread',				0,											''),
				array('pm_replied',			0,											''),
				array('pm_marked',				0,											''),
				array('pm_forwarded',			0,											''),
				array('folder_id',				PRIVMSGS_OUTBOX,							''),

				'where'			=> 'pun_pm_messages.read_at = 0',
			),

			// Sentbox
			array(
				'target'		=> (PM_ENABLED) ? PRIVMSGS_TO_TABLE : '',
				'primary'		=> 'pun_pm_messages.id',

				array('msg_id',				'pun_pm_messages.id',						''),
				array('user_id',				'pun_pm_messages.sender_id',				''),
				array('author_id',				'pun_pm_messages.sender_id',				''),
				array('pm_deleted',			0,											''),
				array('pm_new',				0,											''),
				array('pm_unread',				0,											''),
				array('pm_replied',			0,											''),
				array('pm_marked',				0,											''),
				array('pm_forwarded',			0,											''),
				array('folder_id',				PRIVMSGS_SENTBOX,							''),

				'where'			=> 'pun_pm_messages.read_at <> 0',
			),

			array(
				'target'		=> GROUPS_TABLE,
				'autoincrement'	=> 'group_id',
				'query_first'	=> array('target', $convert->truncate_statement . GROUPS_TABLE),

				array('group_id',				'groups.g_id',						''),
				array('group_type',			GROUP_CLOSED,						''),
				array('group_display',			0,									''),
				array('group_legend',			0,									''),
				array('group_rank',			'groups.g_user_title',				'punbb_rank_id_from_name'),
				array('group_name',			'groups.g_title',					'punbb_convert_group_name'), // punbb_convert_group_name will run htmlspecialchars 
				array('group_desc',			'',									''),
			),

			array(
				'target'		=> USER_GROUP_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . USER_GROUP_TABLE),
				'execute_first'	=> '
					add_default_groups();
				',

				array('group_id',		'users.group_id',					''),
				array('user_id',		'users.id',							''),
				array('group_leader',	0,									''),
				array('user_pending',	0,									''),

				'where'			=> 'users.id <> 1',
			),

			array(
				'target'		=> USERS_TABLE,
				'primary'		=> 'users.id',
				'autoincrement'	=> 'user_id',
				'query_first'	=> array(
					array('target', 'DELETE FROM ' . USERS_TABLE . ' WHERE user_id <> ' . ANONYMOUS),
					array('target', $convert->truncate_statement . BOTS_TABLE)
				),

				'execute_last'	=> '
					remove_invalid_users();
				',

				array('user_id',				'users.id',							''),
				array('',						'users.id AS poster_id',			''),
				array('user_type',				'users.group_id',					'set_user_type'),
				array('group_id',				'users.group_id',					'punbb_set_primary_group'),
				array('user_regdate',			'users.registered',					''),
				array('username',				'users.username',					'punbb_htmlspecialchars'),
				array('username_clean',		'users.username',					array('function1' => 'punbb_htmlspecialchars', 'function2' => 'utf8_clean_string')),
				array('user_password',			'users.password',					'phpbb_hash'),
				array('user_pw_salt',			'users.salt',						'punbb_pw_salt'),
				array('user_pass_convert',		1,									''),
				array('user_posts',			'users.num_posts',					''),
				array('user_email',			'users.email',						array('function1' => 'punbb_htmlspecialchars', 'function2' => 'strtolower')),
				array('user_email_hash',		'users.email',						array('function1' => 'punbb_htmlspecialchars', 'function2' => 'gen_email_hash')),
				array('user_birthday',			'',									''),
				array('user_lastvisit',		'users.last_visit',					''),
				array('user_lastmark',			'users.last_visit',					''),
				array('user_lang',				$config['default_lang'],			''),
				array('user_timezone',			'users.timezone',					'floatval'),
				array('user_dst',				'users.dst',						''),
				array('user_dateformat',		'',									'fill_dateformat'),
				array('user_inactive_reason',	'',									'punbb_inactive_reason'),
				array('user_inactive_time',	'',									'punbb_inactive_time'),

				array('user_interests',		'',									''),
				array('user_occ',				'',									''),
				array('user_website',			'users.url',						array('function1' => 'null_to_str',  'function2' => 'validate_website', 'function3' => 'punbb_htmlspecialchars')),
				array('user_jabber',			'users.jabber',						'punbb_htmlspecialchars'),
				array('user_msnm',				'users.msn',						'punbb_htmlspecialchars'),
				array('user_yim',				'users.yahoo',						'punbb_htmlspecialchars'),
				array('user_aim',				'users.aim',						'punbb_htmlspecialchars'),
				array('user_icq',				'users.icq',						'punbb_htmlspecialchars'),
				array('user_from',				'users.location',					'punbb_htmlspecialchars'),
				array('user_rank',				0,									''),
				array('user_permissions',		'',									''),

				array('user_avatar',			'users.avatar',						'punbb_import_avatar'),
				array('user_avatar_type',		AVATAR_UPLOAD,						''),
				array('user_avatar_width',		'',									'punbb_get_avatar_width'),
				array('user_avatar_height',	'',									'punbb_get_avatar_height'),

				array('user_new_privmsg',		0,									''),
				array('user_unread_privmsg',	0,									''),
				array('user_last_privmsg',		0,									''),
				array('user_emailtime',		'users.last_email_sent',			'null_to_zero'),
				array('user_notify',			1,									''),
				array('user_notify_pm',		1,									''),
				array('user_notify_type',		NOTIFY_EMAIL,						''),
				array('user_allow_pm',			1,									''),
				array('user_allow_viewonline',	1,									''),
				array('user_allow_viewemail',	0,									''),
				array('user_actkey',			'users.activate_key',				'null_to_str'),
				array('user_newpasswd',		'',									''),
				array('user_style',			$config['default_style'],			''),

				array('user_options',			'',										'set_user_options'),
				array('',						'users.show_avatars AS viewavatars',	''),
				array('',						'users.show_img AS viewimg',			''),
				array('',						'users.show_smilies AS viewsmilies',	''),
				array('',						'users.show_sig AS viewsigs',			''),

				array('user_sig_bbcode_uid',		'users.registered',					'make_uid'),
				array('user_sig',					'users.signature',					'punbb_prepare_message'),
				array('user_sig_bbcode_bitfield',	'',									'get_bbcode_bitfield'),
				array('',							'users.registered AS post_time',	''),

				'where'		=> 'users.id <> 1',
			),
		),
	);
}

?>
