<?php
/**
*
* @package install
* @version $Id$
* @copyright (c) 2006 phpBB Group
* @copyright (c) 2013 prototech
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

if (!defined('IN_PHPBB'))
{
	exit;
}

/**
* Counterpart to is_empty()
*/
function not_empty($mixed)
{
	return !empty($mixed);
}

/**
* Convert seconds to minutes
*/
function seconds_to_minutes($seconds)
{
	$seconds = (int) $seconds;
	return round($seconds / 60);
}

/**
* Set encoding
*/
function punbb_set_encoding($text)
{
	if (!class_exists('utf_normalizer'))
	{
		include($phpbb_root_path . 'includes/utf/utf_normalizer.' . $phpEx);
	}

	$text = utf8_decode_ncr($text);
	utf_normalizer::nfc($text);

	return $text;
}

/**
* Simple utf8_htmlspecialchars wrapper which ensures that null is not returned
*/
function punbb_htmlspecialchars($text)
{
	if ($text === null || $text === '')
	{
		return '';
	}
	return utf8_htmlspecialchars($text);
}

/**
* Import attachment
*/
function punbb_import_attachment($physical_filename)
{
	global $config, $convert, $convert_row, $phpbb_root_path, $phpEx;

	$target = $convert_row['poster_id'] . '_' . md5(unique_id());
	$extension = strtolower($convert_row['file_ext']);

	import_attachment($physical_filename, $target);

	// We'll go ahead and try to create thumbnails...
	if ($config['img_create_thumbnail'] && in_array($extension, array('png', 'jpg', 'jpeg', 'gif')))
	{
		if (!function_exists('create_thumbnail'))
		{
			include($phpbb_root_path . 'includes/functions_posting.' . $phpEx);
		}

		$source = relative_base($convert->convertor['upload_path'] . $physical_filename);
		$thumb_target = $phpbb_root_path . $config['upload_path'] . '/' . 'thumb_' . $target;
		$result = create_thumbnail($source, $thumb_target, mimetype($convert_row['filename']));

		if ($result)
		{
			$convert_row['has_thumbnail'] = true;
		}
	}

	return $target;
}

function punbb_has_thumbnail()
{
	global $convert_row;

	return (empty($convert_row['has_thumbnail'])) ? 0 : 1;
}

function punbb_update_post_attach_status()
{
	global $db;

	if (!ATTACH_ENABLED)
	{
		return;
	}

	$sql = 'SELECT DISTINCT(post_msg_id)
		FROM ' . ATTACHMENTS_TABLE;
	$result = $db->sql_query($sql);
	$post_ids = array();

	while ($row = $db->sql_fetchrow($result))
	{
		$post_ids[] = (int) $row['post_msg_id'];

		if (sizeof($post_ids) >= 250)
		{
			$sql = 'UPDATE ' . POSTS_TABLE . ' SET post_attachment = 1 WHERE ' . $db->sql_in_set('post_id', $post_ids);
			$db->sql_query($sql);
			$post_ids = array();
		}
	}
	$db->sql_freeresult($result);

	if (!empty($post_ids))
	{
		$sql = 'UPDATE ' . POSTS_TABLE . ' SET post_attachment = 1 WHERE ' . $db->sql_in_set('post_id', $post_ids);
		$db->sql_query($sql);
	}
}

/**
* Determine if the string has more than one ip. Return single ip or empty string if there is more than one ip in the string.
*/
function punbb_handle_ban_ip($ip)
{
	// If we have no ip or multiple ips, return empty string.
	if (!$ip || utf8_strpos($ip, ' ') !== false)
	{
		return '';
	}
	return punbb_htmlspecialchars(decode_ban_ip($ip));
}

/**
* Determine if the provided email is an actual email address or a domain. If a domain add a wildcard to the domain.
*/
function punbb_fix_ban_email($email)
{
	if (empty($email))
	{
		return '';
	}

	if (utf8_strpos($email, '@') === false)
	{
		$email = '*@' . $email;
	}
	return punbb_htmlspecialchars($email);
}

/**
* Import ban list rows where multiple ip's are contained in a single row.
*/
function punbb_import_banlist_extras()
{
	global $convert, $src_db, $db;

	// If the ip field has a space, then it has more than one ip
	$sql = 'SELECT ip, expire, message
		FROM ' . $convert->src_table_prefix . 'bans
		WHERE ip LIKE "% %"';
	$result = $src_db->sql_query($sql);
	$insert_ary = array();

	while ($row = $src_db->sql_fetchrow($result))
	{
		$ips = explode(' ', $row['ip']);
		$ips = array_map('decode_ban_ip', $ips);

		foreach ($ips as $ip)
		{
			$insert_ary[] = array(
				'ban_ip'			=> punbb_htmlspecialchars($ip), // Run htmlspecialchars just to be safe...
				'ban_end'			=> (int) null_to_zero($row['expire']),
				'ban_reason'		=> punbb_htmlspecialchars($row['message']),
				'ban_give_reason'	=> punbb_htmlspecialchars($row['message']),
			);
		}
	}
	$src_db->sql_freeresult($result);

	// Insert any left overs that didn't complete a batch.
	if (!empty($insert_ary))
	{
		$db->sql_multi_insert(BANLIST_TABLE, $insert_ary);
	}
}

/**
* Determine the correct user activation method
*/
function punbb_require_activation()
{
	$reg_enabled = get_config_value('o_regs_allow');
	$user_self = get_config_value('o_regs_verify');

	if ($reg_enabled)
	{
		return USER_ACTIVATION_DISABLE;
	}

	if ($user_self)
	{
		return USER_ACTIVATION_SELF;
	}
	return USER_ACTIVATION_NONE;
}

/**
* Fix dummy moved topic placeholders by filling them with data from the original topic.
*/

function punbb_fix_moved_topics()
{
	global $db;

	$sql = 'SELECT t.topic_id, m.topic_poster, m.topic_views, m.topic_replies, m.topic_replies_real, m.topic_first_post_id, m.topic_first_poster_name,
				m.topic_first_poster_colour, m.topic_last_post_id, m.topic_last_poster_name, m.topic_last_poster_colour, m.topic_last_post_time, m.topic_last_view_time
		FROM ' . TOPICS_TABLE . ' t
		LEFT JOIN ' . TOPICS_TABLE . ' m
			ON (t.topic_moved_id = m.topic_id)
		WHERE t.topic_moved_id <> 0';
	$result = $db->sql_query($sql);

	while ($row = $db->sql_fetchrow($result))
	{
		$topic_id = (int) $row['topic_id'];
		unset($row['topic_id']);

		$sql = 'UPDATE ' . TOPICS_TABLE . ' SET ' . $db->sql_build_array('UPDATE', $row) . ' WHERE topic_id = ' . $topic_id;
		$db->sql_query($sql);
	}
	$db->sql_freeresult($result);
}

/**
* Close multiple open post reports for a single post, leaving only the latest open.
*/
function punbb_post_reports()
{
	global $db;

	$sql = 'SELECT report_id, post_id
		FROM ' . REPORTS_TABLE . '
		WHERE report_closed = 0';
	$result = $db->sql_query($sql);
	$reports = array();

	while ($row = $db->sql_fetchrow($result))
	{
		$post_id = (int) $row['post_id'];

		if (!isset($reports[$post_id]))
		{
			$reports[$post_id] = 0;
		}
		$reports[$post_id] = max($reports[$post_id], $row['report_id']);
	}
	$db->sql_freeresult($result);

	if (!empty($reports))
	{
		// Close the "extra" reports
		$sql = 'UPDATE ' . REPORTS_TABLE . '
			SET report_closed = 1
			WHERE report_closed = 0
				AND ' . $db->sql_in_set('report_id', $reports, true);
		$db->sql_query($sql);

		// Mark the posts as reported
		$sql = 'UPDATE ' . POSTS_TABLE . '
			SET post_reported = 1 
			WHERE ' . $db->sql_in_set('post_id', array_keys($reports));
		$db->sql_query($sql);
	}
}

/**
* Convert authentication
* user, group and forum table has to be filled in order to work
*/
function punbb_convert_authentication($mode)
{
	global $db, $src_db, $same_db, $convert, $user, $config, $cache;

	if ($mode == 'start')
	{
		$db->sql_query($convert->truncate_statement . ACL_USERS_TABLE);
		$db->sql_query($convert->truncate_statement . ACL_GROUPS_TABLE);

		// We will only assign permission roles to the default phpBB groups to ensure that things work. We will not be dealing with forum-based permissions at all.
		// After conversion the administrator will have to assign permissions as needed.


		// Grab user ids of users in the administrators group
		$sql = "SELECT id
			FROM {$convert->src_table_prefix}users
			WHERE group_id = 1
			ORDER BY registered ASC";
		$result = $src_db->sql_query($sql);

		while ($row = $src_db->sql_fetchrow($result))
		{
			$user_id = (int) $row['id'];
			// Set founder admin...
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_type = ' . USER_FOUNDER . "
				WHERE user_id = $user_id";
			$db->sql_query($sql);
		}
		$src_db->sql_freeresult($result);

		$sql = 'SELECT group_id
			FROM ' . GROUPS_TABLE . "
			WHERE group_name = '" . $db->sql_escape('BOTS') . "'";
		$result = $db->sql_query($sql);
		$bot_group_id = (int) $db->sql_fetchfield('group_id');
		$db->sql_freeresult($result);

		user_group_auth('guests', 'SELECT user_id, {GUESTS} FROM ' . USERS_TABLE . ' WHERE user_id = ' . ANONYMOUS, false);
		user_group_auth('registered', 'SELECT user_id, {REGISTERED} FROM ' . USERS_TABLE . ' WHERE user_id <> ' . ANONYMOUS . " AND group_id <> $bot_group_id", false);

		$auth_sql = 'SELECT id AS user_id, {ADMINISTRATORS} FROM ' . $convert->src_table_prefix . 'users WHERE group_id = 1';
		user_group_auth('administrators', $auth_sql, true);

		// We'll only put administrators in the Global Moderators group. We won't do so for PunBB's Moderators group as the group can be deleted, 
		// thus we may inadvertantly give users unwanted privileges.
		$auth_sql = 'SELECT id AS user_id, {GLOBAL_MODERATORS} FROM ' . $convert->src_table_prefix . 'users WHERE group_id = 1';
		user_group_auth('global_moderators', $auth_sql, true);
	}
	else if ($mode == 'first')
	{
		// Assign permission roles and other default permissions

		// guests having u_download and u_search ability
		$db->sql_query('INSERT INTO ' . ACL_GROUPS_TABLE . ' (group_id, forum_id, auth_option_id, auth_role_id, auth_setting) SELECT ' . get_group_id('guests') . ', 0, auth_option_id, 0, 1 FROM ' . ACL_OPTIONS_TABLE . " WHERE auth_option IN ('u_', 'u_download', 'u_search')");

		// administrators/global mods having full user features
		mass_auth('group_role', 0, 'administrators', 'USER_FULL');
		mass_auth('group_role', 0, 'global_moderators', 'USER_FULL');

		// By default all converted administrators are given full access
		mass_auth('group_role', 0, 'administrators', 'ADMIN_FULL');

		// All registered users are assigned the standard user role
		mass_auth('group_role', 0, 'registered', 'USER_STANDARD');
		mass_auth('group_role', 0, 'registered_coppa', 'USER_STANDARD');

		// Instead of administrators being global moderators we give the MOD_FULL role to global mods (admins already assigned to this group)
		mass_auth('group_role', 0, 'global_moderators', 'MOD_FULL');
	}
}

/**
* Determine if the poster is a guest and return the poster name. If a registered user, returns an empty string.
*/
function punbb_guest_name($poster_name)
{
	global $convert_row;

	if ($convert_row['poster_id'] == 1)
	{
		return punbb_htmlspecialchars($poster_name);
	}
	return '';
}

/**
* Determine if the current post row is the first post of the topic. If so, return the topic subject as the post subject.
*/
function punbb_post_subject($topic_subject)
{
	global $convert_row;

	if ($convert_row['first_post_id'] != $convert_row['post_id'])
	{
		return '';
	}
	return punbb_htmlspecialchars($topic_subject);
}

/**
* Format PM to address
*/
function punbb_pm_to_user($user_id)
{
	return 'u_' . (int) $user_id;
}

/**
* Convert the group name, making sure to avoid conflicts with 3.0 special groups
*/
function punbb_convert_group_name($group_name)
{
	$default_groups = array(
		'GUESTS',
		'REGISTERED',
		'REGISTERED_COPPA',
		'GLOBAL_MODERATORS',
		'ADMINISTRATORS',
		'BOTS',
	);

	if (in_array(strtoupper($group_name), $default_groups))
	{
		$group_name = 'PunBB - ' . $group_name;
	}
	return punbb_htmlspecialchars($group_name);
}

function punbb_rank_id_from_name($rank_name)
{
	global $db;

	if (!$rank_name)
	{
		return 0;
	}
	$rank_name = punbb_htmlspecialchars($rank_name);

	$sql = 'SELECT rank_id
		FROM ' . RANKS_TABLE . '
		WHERE rank_title = "' . $db->sql_escape($rank_name) . '"';
	$result = $db->sql_query_limit($sql, 1);
	return (int) $db->sql_fetchfield('rank_id');
}

/**
* Empty function to prevent errors when including the message parser file from PunBB
*/
function get_hook()
{
}

/**
* Convert smilies. PunBB doesn't have the ability to configure smilies, but hard-codes them in the message parser instead.
* We will try to convert them using the hard-coded array in the file where the parser is located. *cross your fingers*
*/
function punbb_convert_smilies()
{
	global $db, $convert, $phpEx;

	// Clear out the smilies tables to avoid conflicts...
	$db->sql_query($convert->truncate_statement . SMILIES_TABLE);

	// Define FORUM so that the file doesn't exit on us...
	define('FORUM', true);

	// Include the file where the parser is located
	if (!function_exists('do_bbcode'))
	{
		include($convert->options['forum_path'] . '/include/parser.' . $phpEx);
	}

	// Do we have any smilies in the array?
	if (empty($smilies))
	{
		return;
	}

	$insert_ary = array();
	$i = 1;

	foreach ($smilies as $code => $filename)
	{
		import_smiley($filename);

		$insert_ary[] = array(
			'code'					=> $code,
			'emotion'				=> '',
			'smiley_url'			=> $filename,
			'smiley_width'			=> get_smiley_width($filename),
			'smiley_height'			=> get_smiley_height($filename),
			'smiley_order'			=> $i,
			'display_on_posting'	=> 1,		
		);
	}

	if (!empty($insert_ary))
	{
		$db->sql_multi_insert(SMILIES_TABLE, $insert_ary);
	}
}

/**
* Set primary group
*/
function punbb_set_primary_group($group_id)
{
	// Unactivated users have a group id of 0 in PunBB
	if (!$group_id)
	{
		return 0;
	}
	else if ($group_id == 1)
	{
		return get_group_id('administrators');
	}
	else
	{
		return get_group_id('registered');
	}
}

/**
* Get the salt for the password
*/
function punbb_pw_salt($salt)
{
	global $convert_row;

	// If we have no salt, then a previous version on PunBB was used at some point...
	if (!$salt)
	{
		if (strlen($convert_row['password']) == 40)
		{
			return 'sha1_only';
		}
		return 'md5_only';
	}
	return $salt;
}

/**
* Get the avatar filename for the current user row.
*/
function punbb_avatar_filename()
{
	global $convert_row;

	$ext_map = array(
		1	=> 'gif',
		2	=> 'jpg',
		3	=> 'png'
	);

	if (!$convert_row['avatar'] || !isset($ext_map[$convert_row['avatar']]))
	{
		return '';
	}
	return $convert_row['id'] . '.' . $ext_map[$convert_row['avatar']];
}

/**
* Transfer avatars, copying the image if it was uploaded
*/
function punbb_import_avatar()
{
	global $convert_row;

	$user_avatar = punbb_avatar_filename();

	if ($user_avatar)
	{
		return import_avatar($user_avatar, false, $convert_row['id']);
	}
	return '';
}


/**
* Get avatar height
*/
function punbb_get_avatar_height($user_avatar)
{
	$user_avatar = punbb_avatar_filename();

	if ($user_avatar)
	{
		return get_avatar_height($user_avatar);
	}
	return 0;
}


/**
* Get avatar width
*/
function punbb_get_avatar_width($user_avatar)
{
	$user_avatar = punbb_avatar_filename();

	if ($user_avatar)
	{
		return get_avatar_width($user_avatar);
	}
	return 0;
}

/**
* Calculate the date a user became inactive
*/
function punbb_inactive_time()
{
	global $convert_row;

	if ($convert_row['group_id'])
	{
		return 0;
	}

	return (int) $convert_row['registered'];
}

/**
* Calculate the reason a user became inactive
*/
function punbb_inactive_reason()
{
	global $convert_row;

	if ($convert_row['group_id'])
	{
		return 0;
	}

	return INACTIVE_REGISTER;
}

/**
* Set forum flags - only prune old polls by default
*/
function punbb_forum_flags()
{
	global $config;

	// Set forum flags
	$forum_flags = 0;

	// FORUM_FLAG_LINK_TRACK
	$forum_flags += 0;

	// FORUM_FLAG_PRUNE_POLL
	$forum_flags += FORUM_FLAG_PRUNE_POLL;

	// FORUM_FLAG_PRUNE_ANNOUNCE
	$forum_flags += 0;

	// FORUM_FLAG_PRUNE_STICKY
	$forum_flags += 0;

	// FORUM_FLAG_ACTIVE_TOPICS
	$forum_flags += 0;

	// FORUM_FLAG_POST_REVIEW
	$forum_flags += FORUM_FLAG_POST_REVIEW;

	// FORUM_FLAG_QUICK_REPLY
	$forum_flags += ($config['allow_quick_reply']) ? FORUM_FLAG_QUICK_REPLY : 0;

	return $forum_flags;
}

/**
* Insert/Convert forums
*/
function punbb_insert_forums()
{
	global $db, $src_db, $same_db, $convert, $user, $config;

	$db->sql_query($convert->truncate_statement . FORUMS_TABLE);

	// Determine the highest id used within the old forums table (we add the categories after the forum ids)
	$sql = 'SELECT MAX(id) AS max_forum_id
		FROM ' . $convert->src_table_prefix . 'forums';
	$result = $src_db->sql_query($sql);
	$max_forum_id = (int) $src_db->sql_fetchfield('max_forum_id');
	$src_db->sql_freeresult($result);

	$max_forum_id++;

	// Insert categories
	$sql = 'SELECT id, cat_name
		FROM ' . $convert->src_table_prefix . 'categories
		ORDER BY disp_position';

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'binary'");
	}

	$result = $src_db->sql_query($sql);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'utf8'");
	}

	switch ($db->sql_layer)
	{
		case 'mssql':
		case 'mssql_odbc':
		case 'mssqlnative':
			$db->sql_query('SET IDENTITY_INSERT ' . FORUMS_TABLE . ' ON');
		break;
	}

	$cats_added = $cat_tree_ids = array();

	while ($row = $src_db->sql_fetchrow($result))
	{
		$cat_sql_arys[$row['id']] = array(
			'forum_id'		=> (int) $max_forum_id,
			'forum_name'	=> punbb_htmlspecialchars($row['cat_name']),
			'parent_id'		=> 0,
			'forum_parents'	=> '',
			'forum_desc'	=> '',
			'forum_type'	=> FORUM_CAT,
			'forum_status'	=> ITEM_UNLOCKED,
			'forum_rules'	=> '',
		);

		$cat_tree_ids[$row['id']] = array();
		$cats_added[$row['id']] = $max_forum_id;
		$max_forum_id++;
	}
	$src_db->sql_freeresult($result);

	// Now insert the forums
	$sql = 'SELECT  * FROM ' . $convert->src_table_prefix . 'forums
		ORDER BY cat_id, disp_position';

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'binary'");
	}

	$result = $src_db->sql_query($sql);

	if ($convert->mysql_convert && $same_db)
	{
		$src_db->sql_query("SET NAMES 'utf8'");
	}

	$node = $last_cat = 0;

	while ($row = $src_db->sql_fetchrow($result))
	{
		if (!isset($cats_added[$row['cat_id']]))
		{
			continue;
		}

		if ($last_cat != $row['cat_id'])
		{
			if ($last_cat)
			{
				$cat_tree_ids[$last_cat]['right_id'] = $node++;
			}
			$cat_tree_ids[$row['cat_id']]['left_id'] = $node++;
		}

		// Define the new forums sql ary
		$sql_ary = array(
			'forum_id'			=> (int) $row['id'],
			'forum_name'		=> punbb_htmlspecialchars($row['forum_name']),
			'parent_id'			=> (int) $cats_added[$row['cat_id']],
			'forum_parents'		=> '',
			'forum_desc'		=> punbb_htmlspecialchars($row['forum_desc']),
			'forum_type'		=> ($row['redirect_url'] == null) ? FORUM_POST : FORUM_LINK,
			'forum_link'		=> punbb_htmlspecialchars(null_to_str($row['redirect_url'])),
			'forum_status'		=> ITEM_UNLOCKED,
			'enable_prune'		=> 0,
			'prune_next'		=> 0,
			'prune_days'		=> 0,
			'prune_viewed'		=> 0,
			'prune_freq'		=> 0,
			'left_id'			=> $node++,
			'right_id'			=> $node++,

			'forum_flags'		=> punbb_forum_flags(),
			'forum_options'		=> 0,

			// Default values
			'forum_desc_bitfield'		=> '',
			'forum_desc_options'		=> 7,
			'forum_desc_uid'			=> '',
			'forum_password'			=> '',
			'forum_style'				=> 0,
			'forum_image'				=> '',
			'forum_rules'				=> '',
			'forum_rules_link'			=> '',
			'forum_rules_bitfield'		=> '',
			'forum_rules_options'		=> 7,
			'forum_rules_uid'			=> '',
			'forum_topics_per_page'		=> 0,
			'forum_posts'				=> 0,
			'forum_topics'				=> 0,
			'forum_topics_real'			=> 0,
			'forum_last_post_id'		=> 0,
			'forum_last_poster_id'		=> 0,
			'forum_last_post_subject'	=> '',
			'forum_last_post_time'		=> 0,
			'forum_last_poster_name'	=> '',
			'forum_last_poster_colour'	=> '',
			'display_on_index'			=> 1,
			'enable_indexing'			=> 1,
			'enable_icons'				=> 0,
		);
		$last_cat = (int) $row['cat_id'];

		$sql = 'INSERT INTO ' . FORUMS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
	}
	$src_db->sql_freeresult($result);

	$cat_tree_ids[$last_cat]['right_id'] = $node; 

	foreach ($cat_sql_arys as $cat_id => $sql_ary)
	{
		$sql_ary = array_merge($sql_ary, $cat_tree_ids[$cat_id]);

		$sql = 'INSERT INTO ' . FORUMS_TABLE . ' ' . $db->sql_build_array('INSERT', $sql_ary);
		$db->sql_query($sql);
	}
	unset($cat_sql_arys);

	switch ($db->sql_layer)
	{
		case 'postgres':
			$db->sql_query("SELECT SETVAL('" . FORUMS_TABLE . "_seq',(select case when max(forum_id)>0 then max(forum_id)+1 else 1 end from " . FORUMS_TABLE . '));');
		break;

		case 'mssql':
		case 'mssql_odbc':
		case 'mssqlnative':
			$db->sql_query('SET IDENTITY_INSERT ' . FORUMS_TABLE . ' OFF');
		break;

		case 'oracle':
			$result = $db->sql_query('SELECT MAX(forum_id) as max_id FROM ' . FORUMS_TABLE);
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			$largest_id = (int) $row['max_id'];

			if ($largest_id)
			{
				$db->sql_query('DROP SEQUENCE ' . FORUMS_TABLE . '_seq');
				$db->sql_query('CREATE SEQUENCE ' . FORUMS_TABLE . '_seq START WITH ' . ($largest_id + 1));
			}
		break;
	}
}

/**
* Reparse the message stripping out the bbcode_uid values and adding new ones and setting the bitfield
* @todo What do we want to do about HTML in messages - currently it gets converted to the entities, but there may be some objections to this
*/
function punbb_prepare_message($message)
{
	global $phpbb_root_path, $phpEx, $db, $convert, $user, $config, $cache, $convert_row, $message_parser;

	if (!$message)
	{
		$convert->row['mp_bbcode_bitfield'] = $convert_row['mp_bbcode_bitfield'] = 0;
		return '';
	}

	if (strpos($message, '[quote=') !== false)
	{
		$message = preg_replace('/\[quote="?(.*?)"?\]/s', '[quote=&quot;\1&quot;]', $message);
		$message = preg_replace('/\[quote=\\\"(.*?)\\\"\]/s', '[quote=&quot;\1&quot;]', $message);

		// let's hope that this solves more problems than it causes. Deal with escaped quotes.
		$message = str_replace('\"', '&quot;', $message);
		$message = str_replace('\&quot;', '&quot;', $message);
	}

	$message = str_replace('[list=*]', '[list]', $message);

	// Already the new user id ;)
	$user_id = $convert->row['poster_id'];

	$message = str_replace('<br />', "\n", $message);
	$message = str_replace('<', '&lt;', $message);
	$message = str_replace('>', '&gt;', $message);

	$message_parser->warn_msg = array(); // Reset the errors from the previous message
	$message_parser->bbcode_uid = make_uid($convert->row['post_time']);
	$message_parser->message = $message;
	unset($message);

	// Make sure options are set.
	$enable_bbcode = (!isset($convert->row['registered'])) ? $config['allow_bbcode'] : $config['allow_sig_bbcode'];
	$enable_smilies = (!isset($convert->row['registered'])) ? ($config['allow_smilies'] && empty($convert_row['hide_smilies'])) : $config['allow_sig_smilies'];
	$enable_magic_url = (bool) get_config_value('make_links');

	// parse($allow_bbcode, $allow_magic_url, $allow_smilies, $allow_img_bbcode = true, $allow_flash_bbcode = true, $allow_quote_bbcode = true, $allow_url_bbcode = true, $update_this_message = true, $mode = 'post')
	$message_parser->parse($enable_bbcode, $enable_magic_url, $enable_smilies);

	if (sizeof($message_parser->warn_msg))
	{
		$msg_id = isset($convert->row['post_id']) ? $convert->row['post_id'] : $convert->row['id'];
		$convert->p_master->error('<span style="color:red">' . $user->lang['POST_ID'] . ': ' . $msg_id . ' ' . $user->lang['CONV_ERROR_MESSAGE_PARSER'] . ': <br /><br />' . implode('<br />', $message_parser->warn_msg), __LINE__, __FILE__, true);
	}

	$convert->row['mp_bbcode_bitfield'] = $convert_row['mp_bbcode_bitfield'] = $message_parser->bbcode_bitfield;

	$message = $message_parser->message;
	unset($message_parser->message);

	return $message;
}

/**
* Return the bitfield calculated by the previous function
*/
function get_bbcode_bitfield()
{
	global $convert_row;

	return $convert_row['mp_bbcode_bitfield'];
}

/**
* Add user_pw_salt column to users table.
*/
function punbb_add_login_field()
{
	global $db, $phpbb_root_path, $phpEx;

	if (!class_exists('phpbb_db_tools'))
	{
		include($phpbb_root_path . 'includes/db/db_tools.' . $phpEx);
	}

	$schema_changes = array(
		'add_columns'	=> array(
			USERS_TABLE	 => array(
				'user_pw_salt'	=> array('VCHAR:12', ''),
			)
		)
	);

	$db_tools = new phpbb_db_tools($db);
	$db_tools->perform_schema_changes($schema_changes);
}

/**
* Checks whether there are any usernames on the old board that would map to the same
* username_clean on phpBB3. Prints out a list if any exist and exits.
*/
function phpbb_create_userconv_table()
{
	global $db, $src_db, $convert, $table_prefix, $user, $lang;

	$map_dbms = '';
	switch ($db->sql_layer)
	{
		case 'mysql':
			$map_dbms = 'mysql_40';
		break;

		case 'mysql4':
			if (version_compare($db->sql_server_info(true), '4.1.3', '>='))
			{
				$map_dbms = 'mysql_41';
			}
			else
			{
				$map_dbms = 'mysql_40';
			}
		break;

		case 'mysqli':
			$map_dbms = 'mysql_41';
		break;

		case 'mssql':
		case 'mssql_odbc':
		case 'mssqlnative':
			$map_dbms = 'mssql';
		break;

		default:
			$map_dbms = $db->sql_layer;
		break;
	}

	// create a temporary table in which we store the clean usernames
	$drop_sql = 'DROP TABLE ' . USERCONV_TABLE;
	switch ($map_dbms)
	{
		case 'firebird':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id INTEGER NOT NULL,
				username_clean VARCHAR(255) CHARACTER SET UTF8 DEFAULT \'\' NOT NULL COLLATE UNICODE
			)';
		break;

		case 'mssql':
			$create_sql = 'CREATE TABLE [' . USERCONV_TABLE . '] (
				[user_id] [int] NOT NULL ,
				[username_clean] [varchar] (255) DEFAULT (\'\') NOT NULL
			)';
		break;

		case 'mysql_40':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id mediumint(8) NOT NULL,
				username_clean blob NOT NULL
			)';
		break;

		case 'mysql_41':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id mediumint(8) NOT NULL,
				username_clean varchar(255) DEFAULT \'\' NOT NULL
			) CHARACTER SET `utf8` COLLATE `utf8_bin`';
		break;

		case 'oracle':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id number(8) NOT NULL,
				username_clean varchar2(255) DEFAULT \'\'
			)';
		break;

		case 'postgres':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id INT4 DEFAULT \'0\',
				username_clean varchar_ci DEFAULT \'\' NOT NULL
			)';
		break;

		case 'sqlite':
			$create_sql = 'CREATE TABLE ' . USERCONV_TABLE . ' (
				user_id INTEGER NOT NULL DEFAULT \'0\',
				username_clean varchar(255) NOT NULL DEFAULT \'\'
			)';
		break;
	}

	$db->sql_return_on_error(true);
	$db->sql_query($drop_sql);
	$db->sql_return_on_error(false);
	$db->sql_query($create_sql);
}

function phpbb_check_username_collisions()
{
	global $db, $src_db, $convert, $table_prefix, $user, $lang;

	// now find the clean version of the usernames that collide
	$sql = 'SELECT username_clean
		FROM ' . USERCONV_TABLE .'
		GROUP BY username_clean
		HAVING COUNT(user_id) > 1';
	$result = $db->sql_query($sql);

	$colliding_names = array();
	while ($row = $db->sql_fetchrow($result))
	{
		$colliding_names[] = $row['username_clean'];
	}
	$db->sql_freeresult($result);

	// there was at least one collision, the admin will have to solve it before conversion can continue
	if (sizeof($colliding_names))
	{
		$sql = 'SELECT user_id, username_clean
			FROM ' . USERCONV_TABLE . '
			WHERE ' . $db->sql_in_set('username_clean', $colliding_names);
		$result = $db->sql_query($sql);
		unset($colliding_names);

		$colliding_user_ids = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$colliding_user_ids[(int) $row['user_id']] = $row['username_clean'];
		}
		$db->sql_freeresult($result);

		$sql = 'SELECT username, id, num_posts
			FROM ' . $convert->src_table_prefix . 'users
			WHERE ' . $src_db->sql_in_set('id', array_keys($colliding_user_ids));
		$result = $src_db->sql_query($sql);

		$colliding_users = array();
		while ($row = $src_db->sql_fetchrow($result))
		{
			$row['id'] = (int) $row['id'];
			if (isset($colliding_user_ids[$row['id']]))
			{
				$colliding_users[$colliding_user_ids[$row['id']]][] = $row;
			}
		}
		$src_db->sql_freeresult($result);
		unset($colliding_user_ids);

		$list = '';
		foreach ($colliding_users as $username_clean => $users)
		{
			$list .= sprintf($user->lang['COLLIDING_CLEAN_USERNAME'], $username_clean) . "<br />\n";
			foreach ($users as $i => $row)
			{
				$list .= sprintf($user->lang['COLLIDING_USER'], $row['id'], punbb_htmlspecialchars($row['username']), $row['num_posts']) . "<br />\n";
			}
		}

		$lang['INST_ERR_FATAL'] = $user->lang['CONV_ERR_FATAL'];
		$convert->p_master->error('<span style="color:red">' . $user->lang['COLLIDING_USERNAMES_FOUND'] . '</span></b><br /><br />' . $list . '<b>', __LINE__, __FILE__);
	}

	$drop_sql = 'DROP TABLE ' . USERCONV_TABLE;
	$db->sql_query($drop_sql);
}

?>
